classdef response
    %RESPONSE the response of species j to the wave
    properties
        % species properties
        m % mass
        q % charge
        n0 % number density
        T_perp % perpendicular temperature
        T_para % parallel temperature
        wp % plasma frequency
        v_para % parallel thermal velocity
        v_perp % perpendicular thermal velocity
        v0 % plasma drift velocity

        % wave properties
        k_para % parallel wave number
        k_perp % perpendicular wave number
        w % angular frequency
        
        % response properties
        wc % cyclotron frequency
        B % magnetic field
        rhoL % Larmor radius
        lambda % FLR parameter
        eps % charge sign
        In % preprocessed Bessel In
        In_pr % In'
        zetan % preprocessed
        Zn % preprocessed
        Zn_pr % Zn'
        An
        Bn
        Cn

        % not going to sum over n=-inf to inf, but rather from -n_bound to n_bound
        n_bound
    end
    
    methods
        function j = response(s, wv, B, n_bound)
            arguments
                s species
                wv wave
                B double
                n_bound int32
            end
            % copy species properties
            j.m = s.m;
            j.q = s.q;
            j.n0 = s.n0;
            j.T_perp = s.T_perp;
            j.T_para = s.T_para;
            j.v_para = s.v_para;
            j.v_perp = s.v_perp;
            j.v0 = s.v0;
            j.wp = s.wp;

            % copy wave properties
            j.k_para = wv.k_para;
            j.k_perp = wv.k_perp;
            j.w = wv.w;

            % calculate response properties
            j.B = B;
            j.wc = abs(s.q * B / s.m); % in literature, the wc is always positive
            j.rhoL = s.v_perp / j.wc;
            j.lambda = 0.5 * wv.k_perp^2 * j.rhoL^2;
            j.eps = sign(s.q); % eps shows the cyclotron direction
            j.n_bound = n_bound;

            % preprocess bessel In and In'
            n = -n_bound:n_bound;
            j.In = besseli(double(n), j.lambda);
            j.In_pr = besseli_prime(double(n), j.lambda);

            % preprocess An Bn Cn
            j.zetan = arrayfun(@j.getZeta, double(n));
            j.Zn = arrayfun(@zfunc, j.zetan);
            j.Zn_pr = arrayfun(@zfunc_prime, j.zetan);
            
            j.An = arrayfun(@j.getAn, n);
            j.Bn = arrayfun(@j.getBn, n);
            j.Cn = arrayfun(@j.getCn, n);
        end

        % here j denotes the j-th species, aligned with notations in the paper.
        % see https://doi.org/10.1088/1741-4326/ab1ab7.
        function zeta = getZeta(j, n)
            zeta = (j.w + n*j.wc - j.k_para*j.v0)/(j.k_para*j.v_para);
        end

        function An = getAn(j, n)
            l = n + j.n_bound + 1; % from 1 to 2*n_bound+1
            An = (1 - j.k_para*j.v0/j.w)*j.Zn(l) ...
                + j.k_para*j.v_para/j.w*(1 - j.T_perp/j.T_para)*j.Zn_pr(l)*0.5;
        end

        function Bn = getBn(j, n)
            l = n + j.n_bound + 1;
            n = double(n);
            Bn = (1 + n.*j.wc/j.w*(1-j.T_para/j.T_perp))*j.Zn_pr(l) ...
                + 2*n*j.wc*j.T_para*j.v0/(j.w*j.T_perp*j.v_para).*(j.Zn(l) + j.k_para*j.v_para/(j.w+n*j.wc));
        end

        function Cn = getCn(j, n)
            l = n + j.n_bound + 1;
            n = double(n);
            Cn = n*j.wc*j.v0/(j.w*j.v_para)*j.Zn(l) ...
                + (j.T_perp/j.T_para - n*j.wc/j.w*(1 - j.T_perp/j.T_para)) * j.Zn_pr(l)*0.5;
        end

        function [K0_elem, K1_elem, K2_elem,...
                  K3_elem, K4_elem, K5_elem] = K_elems(j) 
            % K_elem: terms immediately after Sum_j
            % see https://doi.org/10.1088/1741-4326/ab1ab7, eq (4)
            % An(n) is an array of 2*n_bound+1
            n = double(-j.n_bound:j.n_bound);

            coef0 = j.wp^2 * exp(-j.lambda) / (j.w * j.k_para * j.v_para);
            K0_elem = coef0 .* sum(j.lambda .* (j.In - j.In_pr) .* j.An);

            coef1 = j.wp^2 * exp(-j.lambda) ./ (j.w .* j.k_para .* j.v_para);
            K1_elem = coef1 .* sum(n.^2 .* j.In ./ j.lambda .* j.An);

            coef2 = j.eps * j.wp^2 * exp(-j.lambda) / (j.w * j.k_para * j.v_para);
            K2_elem = coef2 .* sum(n .* (j.In-j.In_pr) .* j.An);

            coef3 = j.wp^2 * exp(-j.lambda) / (j.w * j.k_para * j.v_para);
            K3_elem = coef3 .* sum(j.In .* (j.w + n.*j.wc)./(j.k_para * j.v_para) .* j.Bn);

            coef4 = j.k_perp * j.wp^2 * exp(-j.lambda) / (j.k_para * j.w * j.wc);
            K4_elem = coef4 .* sum(n .* j.In ./ j.lambda .* j.Cn);

            coef5 = j.k_perp * j.eps * j.wp^2 * exp(-j.lambda) / (j.k_para * j.w * j.wc);
            K5_elem = coef5 .* sum((j.In-j.In_pr) .* j.Cn);
        end
    end
end

